from friendly_traceback.typing_info import InclusionChoice, Info
def parseable(info: Info, include: InclusionChoice = "friendly_tb") -> str:
    """Formatter that separates different parts of the error message for easy parsing."""
    sections = []

    if "header" in info:
        sections.append(f"Header: {info['header']}")

    if "message" in info:
        sections.append(f"Message: {info['message']}")

    if "original_python_traceback" in info:
        sections.append("Original Python Traceback:")
        sections.append(info['original_python_traceback'])

    if "simulated_python_traceback" in info:
        sections.append("Simulated Python Traceback:")
        sections.append(info['simulated_python_traceback'])

    if "shortened_traceback" in info:
        sections.append("Shortened Traceback:")
        sections.append(info['shortened_traceback'])

    if "exception_notes_intro" in info:
        sections.append("Exception Notes Intro:")
        sections.extend(info['exception_notes_intro'])

    if "exception_notes" in info:
        sections.append("Exception Notes:")
        sections.append(info['exception_notes'])

    if "suggest" in info:
        sections.append("Suggestion:")
        sections.append(info['suggest'])

    if "generic" in info:
        sections.append("Generic:")
        sections.append(info['generic'])

    if "parsing_error" in info:
        sections.append("Parsing Error:")
        sections.append(info['parsing_error'])

    if "parsing_error_source" in info:
        sections.append("Parsing Error Source:")
        sections.append(info['parsing_error_source'])

    if "cause" in info:
        sections.append("Cause:")
        sections.append(info['cause'])

    if "detailed_tb" in info:
        sections.append("Detailed Traceback:")
        sections.extend(info['detailed_tb'])

    if "last_call_header" in info:
        sections.append("Last Call Header:")
        sections.append(info['last_call_header'])

    if "last_call_source" in info:
        sections.append("Last Call Source:")
        sections.append(info['last_call_source'])

    if "last_call_variables" in info:
        sections.append("Last Call Variables:")
        sections.append(info['last_call_variables'])

    if "exception_raised_header" in info:
        sections.append("Exception Raised Header:")
        sections.append(info['exception_raised_header'])

    if "exception_raised_source" in info:
        sections.append("Exception Raised Source:")
        sections.append(info['exception_raised_source'])

    if "exception_raised_variables" in info:
        sections.append("Exception Raised Variables:")
        sections.append(info['exception_raised_variables'])

    if "warning_message" in info:
        sections.append("Warning Message:")
        sections.append(info['warning_message'])

    if "warning_location_header" in info:
        sections.append("Warning Location Header:")
        sections.append(info['warning_location_header'])

    if "warning_source" in info:
        sections.append("Warning Source:")
        sections.append(info['warning_source'])

    if "warning_variables" in info:
        sections.append("Warning Variables:")
        sections.append(info['warning_variables'])

    if "additional_variable_warning" in info:
        sections.append("Additional Variable Warning:")
        sections.append(info['additional_variable_warning'])

    if "lang" in info:
        sections.append(f"Language: {info['lang']}")

    if "_tb_data" in info:
        sections.append("Traceback Data:")
        sections.append(str(info['_tb_data']))

    return '\n'.join(sections)


from thonny.languages import tr
import os.path
import re
import subprocess
from logging import getLogger
from typing import Iterable

from thonny import  get_workbench, ui_utils
from thonny.assistance import SubprocessProgramAnalyzer, add_program_analyzer
from thonny.running import get_front_interpreter_for_subprocess

from thonny.config_ui import ConfigurationPage

logger = getLogger(__name__)


class FriendlyAnalyzer(SubprocessProgramAnalyzer):
    def is_enabled(self):
        return get_workbench().get_option("assistance.use_friendly")

    def start_analysis(self, main_file_path, imported_file_paths: Iterable[str]) -> None:
        self.interesting_files = [main_file_path] + list(imported_file_paths)

        args = [
            get_front_interpreter_for_subprocess(),
            "-m",
            "friendly_traceback",
            "--formatter",
            "thonnycontrib.thonny_friendly.parser.parseable",
            "--lang",
            "fr",
#            "--ignore-missing-imports",
#            "--check-untyped-defs",
#            "--warn-redundant-casts",
#            "--warn-unused-ignores",
#            "--show-column-numbers",
            main_file_path,
        ] + list(imported_file_paths)

        logger.debug("Running friendly: %s", " ".join(args))

        # TODO: ignore "... need type annotation" messages



        env = os.environ.copy()
        mypypath = get_workbench().get_option("assistance.mypypath")
        if mypypath:
            env["MYPYPATH"] = mypypath

        self._proc = ui_utils.popen_with_ui_thread_callback(
            args,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            env=env,
            on_completion=self._parse_and_output_warnings,
            # Specify a cwd which is not ancestor of user files.
            # This gives absolute filenames in the output.
            # Note that mypy doesn't accept when cwd is sys.prefix
            # or dirname(sys.executable)
            cwd=os.path.dirname(__file__),
        )

    def _parse_and_output_warnings(self, pylint_proc, out_lines, err_lines):
        if err_lines:
            logger.warning("Friendly: " + "".join(err_lines))
            print("Friendly errors: " + "".join(err_lines))
        if out_lines:
            logger.warning("Friendly out: " + "".join(out_lines))
            print ("Friendly out: " + "".join(out_lines))
        warnings=[]
        file_name = None
        line_number = None
        for line in  "".join(out_lines).split('\n\n') + "".join(err_lines).split('\n\n'):
            atts = {
                "filename": file_name,
                "explanation": None,
                "lineno": line_number,
                "kind": "warning",
                "title": "Warning",
                "msg": "Warning",
                "group": "0",
                "col_offset": 0,
                "symbol": "friendly-warning"
            }
            if "Simulated Python Traceback:" in line:
                match = re.search(r'File "(.*?)", line (\d+)', line)
                if match:
                    file_name = match.group(1)
                    line_number = int(match.group(2))
                    atts['filename'] = file_name
                    atts['lineno'] = line_number
                atts['explanation'] = line.split(':', 1)[1].strip()
                atts['msg'] = "Simulated Python Traceback"
                warnings.append(atts.copy())
            elif "Shortened Traceback:" in line:
                atts['explanation'] = line.split(':', 1)[1].strip()
                atts['msg'] = "Shortened Traceback"
                warnings.append(atts.copy())
            elif "Generic:" in line:
                atts['explanation'] = line.split(':', 1)[1].strip()
                atts['msg'] = "Generic"
                warnings.append(atts.copy())
            elif "Cause:" in line:
                atts['explanation'] = line.split(':', 1)[1].strip()
                atts['msg'] = "Cause"
                warnings.append(atts.copy())
            elif "Exception Raised Header:" in line:
                atts['explanation'] = line.split(':', 1)[1].strip()
                atts['msg'] = "Exception Raised Header"
                warnings.append(atts.copy())
        """ def parse_traceback(output):
            # Define regular expressions to extract file name and line number
            file_pattern = r'File "(.*?)", line (\d+)'
            
            # Search for file name and line number in the traceback
            match = re.search(file_pattern, output)
            if match:
                file_name = match.group(1)
                line_number = int(match.group(2))
                return file_name, line_number
            else:
                return None, None
        file_name, line_number = parse_traceback("".join(out_lines))
        print(file_name, line_number)
        atts = {
                    "filename": file_name,
                    "explanation" : "".join(out_lines),
                    "lineno": line_number,
                    "kind": "a",  # always "error" ?
                    "title": "tiiitle",
                    "msg": "Explication",
                    "group": "0",
                    "helpfull":"helpfull",
                    "confusing":"confusing",
                    "col_offset": 0,
                    "symbol" : "friendly-error"

                }
        warnings.append(atts) """

        """ for line in out_lines:
            m = re.match(r"(.*?):(\d+)(:(\d+))?:(.*?):(.*)", line.strip())
            if m is not None:
                message = m.group(6).strip()
                if message == "invalid syntax":
                    continue  # user will see this as Python error

                filename = m.group(1)
                if filename not in self.interesting_files:
                    logger.warning("MyPy: " + line)
                    continue

                atts = {
                    "filename": filename,
                    "lineno": int(m.group(2)),
                    "kind": m.group(5).strip(),  # always "error" ?
                    "msg": message,
                    "group": "warnings",
                }
                if m.group(3):
                    # https://github.com/thonny/thonny/issues/598
                    atts["col_offset"] = max(int(m.group(4)) - 1, 0)

                # TODO: add better categorization and explanation
                atts["symbol"] = "mypy-" + atts["kind"]
                warnings.append(atts)
            else:
                logger.error("Can't parse MyPy line: " + line.strip())
 """
        self.completion_handler(self, warnings)

class FriendlyConfigPage(ConfigurationPage):
    def __init__(self, master):
        super().__init__(master)
        self.add_checkbox("assistance.use_friendly","Friendly", row=4, columnspan=2,tooltip="Afficher les suggestions Friendly")


def load_plugin():
    add_program_analyzer(FriendlyAnalyzer)
    get_workbench().set_default("assistance.use_friendly", True)
    get_workbench().add_configuration_page("friendly", tr("Friendly"), FriendlyConfigPage, 90)
    




from setuptools import setup
import os.path
import sys

setupdir = os.path.dirname(__file__)

requirements = []
for line in open(os.path.join(setupdir, "requirements.txt"), encoding="UTF-8"):
    if line.strip() and not line.startswith("#"):
        requirements.append(line)

setup(
    author='Selmen Arous',
    author_email='selmen.arous@gmail.com',
    
    url="https://github.com/selmen2004/thonny-friendly",

    
    
    
    platforms=["Windows", "macOS", "Linux"],
    python_requires=">=3.0",
    
    
    py_modules=["thonny_friendly"],
    packages=["thonnycontrib.thonny_friendly"],
)
